package com.cachingapp.cache;

/**
 * This is the main interface in Caching framework. It defines the methods
 * that all the client applications can depend on, to use caching in web portal
 * project. It hides the clients from any 3rd party caching API that is used
 * to implement the caching mechanism (Note: I am currently using Jakarta Turbine's
 * Java Caching System, aka JCS, in the caching framework. We may want to switch to
 * JCache - Java Temporary Caching API (JSR-107) when it becomes available).
 *
 * @version 1.0
 * @author Srini Penchikala
 * @since 10/03/2003
 */

public interface ICacheManager {

   /**
 	* Retrieves a cached object.  Default to look in the cache.
    * @param cacheKey Cache object
    * @param cacheLoader ICacheLoader object
    * @return cached object
    * @throws CachingException
	 */
	 public Object getCacheObject(Object cacheKey, ICacheLoader cacheLoader) throws CachingException;

   /**
    * Retrieves a cached object.  Default to look in the cache and in
    * default cache region.
    * @param cacheKey Cache object
    * @param fromCache Flag whether to get object from cache or not.
    * @param cacheLoader ICacheLoader object
    * @return cached object
    * @throws CachingException
    */
	public Object getCacheObject(Object cacheKey, boolean fromCache, ICacheLoader cacheLoader) throws CachingException;

   /**
    * Retrieves a cached object.  Default to look in the cache for the speficied
    * cache region.
    * @param cacheKey Cache object
    * @param fromCache Flag whether to get object from cache or not.
    * @param cacheLoader ICacheLoader object
    * @return cached object
    * @throws CachingException
    */
	public Object getCacheObject(Object cacheKey, ICacheLoader cacheLoader, String regionName) throws CachingException;

	/**
	 * Retrieves a CacheObject. Second argument decides whether to look
	 * in the cache. Returns a new value object if one can't be
	 * loaded from the database. Database cache synchronization is
	 * handled by removing cache elements upon modification.
    * @param cacheKey Cache object
    * @param fromCache Flag whether to get object from cache or not.
    * @param cacheLoader ICacheLoader object
    * @param regionName name of the cache region
    * @return cached object
    * @throws CachingException
	 */
	public Object getCacheObject(Object cacheKey, boolean fromCache, ICacheLoader cacheLoader, String regionName) throws CachingException;

   /**
    * Stores CacheObjects in database using default cache region.
    * Clears old items and caches new.
    */
	public void putCacheObject(Object cacheKey, Object cacheObject) throws CachingException;

	/**
    * Stores CacheObjects in the specified cache region. Clears old items and caches
    * new.
    */
	public void putCacheObject(Object cacheKey, Object cacheObject, String regionName) throws CachingException;

   /**
    * Stores CacheObjects in the default cache region with the specified
    * day/time to expire the cahed object.
    */
   public void putCacheObject(Object cacheKey, Object cacheObject, boolean expireSchedule) throws CachingException;

   /**
    * Stores CacheObjects in the cache with the specified
    * day/time to expire the cahed object.
    */
   public void putCacheObject(Object cacheKey, Object cacheObject, boolean expireSchedule, String regionName) throws CachingException;

   /**
    * Peeks into the cache using default cache region
    * for a given key without loading the value on demand.
    * If the value is cached, then it will be returned.
    * If the value is not already cached, then null is returned.
    */
	public Object peek(Object cacheKey) throws CachingException;

	/**
	 * Peeks into the cache for a given key without loading
    * the value on demand. If the value is cached, then it
    * will be returned. If the value is not already cached,
    * then null is returned.
	 */
   public Object peek(Object cacheKey, String regionName) throws CachingException;

   /**
    * Peeks into the cache using default cache region
    * for a given key without loading the value on demand.
    * If the value is cached, then it will be returned.
    * If the value is not already cached, then null is returned.
    */
   public boolean isPresent(Object cacheKey) throws CachingException;

	/**
	 * Peeks into the cache for a given key without loading
    * the value on demand. If the value is cached, then it
    * will be returned. If the value is not already cached,
    * then null is returned.
	 */
	public boolean isPresent(Object cacheKey, String regionName) throws CachingException;

	/**
	 * Remove the cached object from default cache region.
	 */
	public void removeCacheObject(Object cacheKey) throws CachingException;

	/**
	 * Remove the cached object from the specified cache region.
	 */
	public void removeCacheObject(Object cacheKey, String regionName) throws CachingException;

	/**
	 * Remove all the cached objects from default cache region.
	 */
   public void clear() throws CachingException;

	/**
	 * Remove all the cached objects from the specified
	 * cache region.
	 */
   public void clear(String regionName) throws CachingException;

	/**
	 * Set the default region cache to null.
	 */
   public void clearCache() throws CachingException;

	/**
	 * Set the specified region cache to null.
	 */
   public void clearCache(String regionName) throws CachingException;


}
// End of ICacheManager.java
